import React from 'react';
import PropTypes from 'prop-types';
import { makeStyles } from '@material-ui/core/styles';

import MUIAppBar from '@material-ui/core/AppBar';
import MUIToolbar from '@material-ui/core/Toolbar';

import IconButton from '../IconButton/IconButton';
import TextLabel from '../TextLabel/TextLabel';
import Menu from '../Menu/Menu';
import MenuItem from '../MenuItem/MenuItem';

import getStyles from './AppBar.styles';

/**
 * Description
 */
const AppBar = (props) => {
    const {
        contextStyles,
        onLogoutClick,
        showMenu,
        testID,
        title,
        toolbar,
    } = props;

    // Override core styles with context styles, separating MUI styles
    const {
        container: containerStyles,
        menuButton: menuButtonStyles,
        menuIcon: menuIconStyles,
        title: titleStyles,
        titleContainer: titleContainerStyles,
        toolbar: toolbarStyles,
        ...muiStyles
    } = getStyles(contextStyles);

    // Create dynamic class names and injected DOM styles for MUI component
    // * makeStyles returns a function
    const styles = makeStyles(muiStyles)();

    // Generate container props
    const containerProps = {
        className: 'c-app-bar__container',
        style: containerStyles, // non-MUI styles
    };
    if (testID) { containerProps['data-test-id'] = `${testID}__app-bar__container`; }

    return (
        <div { ...containerProps }>
            <MUIAppBar
                classes={ {
                    root: styles.bar,
                } }
                position="static">
                <MUIToolbar>
                    { showMenu &&
                        <Menu
                            button={
                                <IconButton
                                    contextStyles={ {
                                        button: menuButtonStyles,
                                        icon: menuIconStyles,
                                    } }
                                    iconName="menu"
                                    onClick={ () => {} } />
                            }>
                            <MenuItem
                                text="Log Out"
                                onClick={ onLogoutClick } />
                        </Menu>
                    }
                    <TextLabel
                        contextStyles={ {
                            container: titleContainerStyles,
                            text: titleStyles,
                        } }>
                        { title }
                    </TextLabel>
                    { toolbar }
                </MUIToolbar>
            </MUIAppBar>
        </div>
    );
};

AppBar.propTypes = {
    /** Styles for this component */
    contextStyles: PropTypes.shape({
        /** Styling for the main container */
        container: PropTypes.object,
    }),

    /** Called on click of the logout menu item */
    onLogoutClick: PropTypes.func,

    /** Id used for testing */
    testID: PropTypes.string,

    /** Title text */
    title: PropTypes.string.isRequired,

    /** Enables display of menu */
    showMenu: PropTypes.bool,

    /** Additional components placed on the right side of the bar */
    toolbar: PropTypes.oneOfType([
        PropTypes.arrayOf(PropTypes.element),
        PropTypes.element,
    ]),
};

AppBar.defaultProps = {
    showMenu: true,
};

export default AppBar;
