import React, { Component } from 'react';
import * as PropTypes from 'prop-types';
import { AgGridReact, AgGridColumn } from 'ag-grid-react';
import DataTableCell from '../DataTableCell/DataTableCell';

import 'ag-grid-community/dist/styles/ag-grid.css';
import 'ag-grid-community/dist/styles/ag-theme-material.css';
import getStyles from './DataTable.styles';

const THEME = 'material';

class DataTable extends Component {
    constructor(props) {
        super(props);
        const {
            cols,
        } = props;

        this.state = {
            columnList: cols,
        };
    }

    render() {
        const {
            columnList,
        } = this.state;
        const {
            contextStyles,
            filterableColumns,
            multiSelectMode,
            resizableColumns,
            rows,
            sortableColumns,
            suppressMovableColumns,
            testID,
        } = this.props;

        const styles = getStyles(contextStyles);

        // Generate container props
        const containerProps = {
            style: styles.container,
            className: 'c-data-table__container',
        };
        if (testID) { containerProps['data-test-id'] = `${testID}__data-table__container`; }

        // Generate viewport props
        const viewportProps = {
            style: styles.viewport,
            className: `ag-theme-${THEME} c-data-table__viewport`,
        };
        if (testID) { containerProps['data-test-id'] = `${testID}__data-table__viewport`; }

        return (
            <div { ...containerProps }>
                <div { ...viewportProps }>
                    <AgGridReact
                        suppressColumnVirtualisation
                        rowBuffer={ 10 }
                        columnDefs={ columnList }
                        defaultColDef={ {
                            columnList,
                            columnStyles: styles.column,
                            filter: filterableColumns,
                            suppressMovable: suppressMovableColumns,
                            resizable: resizableColumns,
                            sortable: sortableColumns,
                        } }
                        onGridReady={ this.handleGridReady }
                        onRowClicked={ this.handleRowClicked }
                        onRowDoubleClicked={ this.handleRowDoubleClicked }
                        rowData={ rows }
                        rowSelection={ multiSelectMode ? 'multiple' : 'single' }
                        suppressCellSelection>
                        { this.renderColumns(columnList) }
                    </AgGridReact>
                </div>
            </div>
        );
    }

    /**
     * Renders columns
     * @param {Object} columnList - meta data about columns
     * @returns {Array}
     */
    renderColumns = (columnList) => (
        columnList.map((column) => (
            <AgGridColumn
                autoHeight
                cellRendererFramework={ column.cellRenderer || DataTableCell }
                field={ column.field }
                filter={ column.filter }
                headerName={ column.headerName }
                key={ column.field }
                pinned={ column.pinned }
                resizable={ column.resizable }
                sortable={ column.sortable }
                suppressMovable={ column.suppressMovable }
                width={ column.width } />
        ))
    );

    /** Makes grid api available to component */
    handleGridReady = (grid) => {
        this.api = grid.api;
    };

    /**
     * Called on click of a row
     * @returns {void}
     */
    handleRowClicked = () => {
        const { onRowClick } = this.props;

        if (onRowClick) {
            onRowClick({
                data: this.api.getSelectedRows(),
                nodes: this.api.getSelectedNodes(),
            });
        }
    };

    /**
     * Called on double click of a row
     * @returns {void}
     */
    handleRowDoubleClicked = () => {
        const { onRowDoubleClick } = this.props;

        if (onRowDoubleClick) {
            onRowDoubleClick({
                data: this.api.getSelectedRows()[0],
                node: this.api.getSelectedNodes()[0],
            });
        }
    };
}

DataTable.propTypes = {
    /** Array containing column definition object */
    cols: PropTypes.arrayOf(PropTypes.shape({
        /**
         * Custom cell renderer
         * @param {Object} params column details for the cell
         */
        cellRenderer: PropTypes.func,

        /** Unique column id which maps to row property */
        field: PropTypes.string,

        /** Enable / disable individual filterable column (overrides global) */
        filter: PropTypes.bool,

        /** Column header display text */
        headerName: PropTypes.string,

        /** Enables column to be pinned */
        pinned: PropTypes.bool,

        /** Enable / disable individual resizable column (overrides global) */
        resizable: PropTypes.bool,

        /** Enable / disable individual sortable column (overrides global) */
        sortable: PropTypes.bool,

        /** Enable / disable individual movable column (overrides global) */
        suppressMovable: PropTypes.bool,

        /** Width of column */
        width: PropTypes.number,
    })).isRequired,

    /** Styles for this component */
    contextStyles: PropTypes.shape({
        /** Column level styles */
        column: PropTypes.object,

        /** Style object for container */
        container: PropTypes.object,

        /** Style object for evenRows*/
        evenRows: PropTypes.object,

        /** Style object for grid filter dialog */
        filter: PropTypes.object,

        /** Style object for grid filter dialog container */
        filterContainer: PropTypes.object,

        /** Style object for grid */
        grid: PropTypes.object,

        /** Style object for header cell */
        headerCell: PropTypes.object,

        /** Style object for header text */
        headerText: PropTypes.object,

        /** Style object for oddRows*/
        oddRows: PropTypes.object,

        /** Style object for grid row */
        row: PropTypes.object,

        /** Style object for selected grid row */
        rowSelected: PropTypes.object,
    }),

    /** Enable / disable global filterable columns */
    filterableColumns: PropTypes.bool,

    /** Enables multiple row selection */
    multiSelectMode: PropTypes.bool,

    /** Called after application of filters */
    // TODO: implement to call server for data refresh
    // onApplyFilters: PropTypes.func,

    /** Called after moving of columns */
    // TODO: implement to save column position after data refresh
    // onColumnMoved: PropTypes.func,

    /** Called after pinning of columns */
    // TODO: implement to save column pinned state after data refresh
    // onColumnPinned: PropTypes.func,

    /** Called after resize of columns */
    // TODO: implement to save column resized state after data refresh
    // onColumnResized: PropTypes.func,

    /**
     * Called on row click
     * @param { { data: Array<Object>, nodes: Array<Object> } }
     */
    // TODO: implement to save row selected state after data refresh
    onRowClick: PropTypes.func,

    /**
     * Called on row double click
     * @param { { data: Object, node: Object } }
     */
    // TODO: implement to save row selected state after data refresh
    onRowDoubleClick: PropTypes.func,

    /**
     * Called after sort request
     * @returns {String} 'asc'|'desc'|''
     * */
    // TODO: implement to call server for data refresh
    // onSortRequested: PropTypes.func,

    /** Enable / disable global sortable columns */
    sortableColumns: PropTypes.bool,

    /** Enable / disable global resizable columns */
    resizableColumns: PropTypes.bool,

    /** Array of objects of column fields */
    rows: PropTypes.arrayOf(PropTypes.object).isRequired,

    /** Enable / disable global movable columns */
    suppressMovableColumns: PropTypes.bool,

    /** ID used for testing */
    testID: PropTypes.string,
};

DataTable.defaultProps = {
};

export default DataTable;
