import React from 'react';
import PropTypes from 'prop-types';
import MuiTextField from '@material-ui/core/TextField';

import { makeStyles } from '@material-ui/core/styles';
import getStyles from './TextField.styles';

const INPUT_TYPES = {
    TEXT: 'text',
    PASSWORD: 'password',
    EMAIL: 'email',
    NUMBER: 'number',
};

/**
 * TextField is a component that allows users to add or modify the input.
 * @see https://material-ui.com/demos/text-fields/
 */
const TextField = (props) => {
    const {
        autoFocus,
        autoGrow,
        contextStyles,
        disabled,
        endAdornment,
        maxLength,
        multiline,
        onChange,
        rowsMax,
        startAdornment,
        testID,
        type,
        value,
    } = props;

    // Override core styles with context styles, separating MUI styles
    const {
        container: containerStyles,
        ...muiStyles
    } = getStyles(contextStyles, autoGrow, value);

    // Create dynamic class names and injected DOM styles for MUI component
    // * makeStyles returns a function
    const styles = makeStyles(muiStyles)();

    // Generate container props
    const containerProps = {
        className: 'c-text-field__container',
        style: containerStyles, // non-MUI styles
    };
    if (testID) { containerProps['data-test-id'] = `${testID}__text-field__container`; }

    // Generate button props
    const inputProps = {
        autoFocus,
        classes: {
            root: styles.input, // MUI styles
        },
        className: 'c-text-field__input',
        disabled,
        InputProps: {
            classes: {
                root: styles.input, // MUI styles
            },
            endAdornment,
            inputProps: { maxLength },
            startAdornment,
        },
        multiline,
        onChange,
        rowsMax,
        type,
        value,
    };
    if (testID) { inputProps['data-test-id'] = `${testID}__text-field__input`; }

    return (
        <div { ...containerProps }>
            <MuiTextField { ...inputProps } />
        </div>
    );
};

TextField.propTypes = {
    /** Focuses the input */
    autoFocus: PropTypes.bool,

    /** Grows/shrinks the input based on the size of its text value */
    autoGrow: PropTypes.bool,

    /** Styles for this component */
    contextStyles: PropTypes.shape({
        /** Styles for the container surrounding the text field */
        container: PropTypes.object,

        /** Styles for the text field */
        input: PropTypes.object,
    }),

    /** Prevents the text field from accepting input */
    disabled: PropTypes.bool,

    /** Elements that needs to be placed towards the end of the TextField */
    endAdornment: PropTypes.node,

    /** Limits the maximum number of characters that can be entered */
    maxLength: PropTypes.number,

    /** Enables the text input to be multiple lines */
    multiline: PropTypes.bool,

    /**
     * Called when the TextField's input changes
     * @param {Event} event - TextField's onChange event
     */
    onChange: PropTypes.func.isRequired,

    /** Maximum number of rows in case of multiline */
    rowsMax: PropTypes.number,

    /** Elements that needs to be placed at the beginning of the TextField */
    startAdornment: PropTypes.node,

    /** Id used for testing */
    testID: PropTypes.string,

    /** Determines the type of input */
    type: PropTypes.oneOf([
        INPUT_TYPES.TEXT,
        INPUT_TYPES.PASSWORD,
        INPUT_TYPES.EMAIL,
        INPUT_TYPES.NUMBER,
    ]),

    /** The text to populate in the input */
    value: PropTypes.string.isRequired,
};

TextField.defaultProps = {
    contextStyles: {},
    type: INPUT_TYPES.TEXT,
};

export default TextField;
export {
    INPUT_TYPES,
};
